<?php
namespace Geral\Util;
use \Geral\Util\Dataset;
class JsonDataset extends Dataset
{
    protected $filepath;

    public function __construct($filepath)
    {
        $this->filepath = $filepath;
        if (!file_exists($filepath)) {
            $elements = array();
            file_put_contents($filepath, json_encode($elements));
        } else {
            $elements = json_decode(file_get_contents($this->filepath), true);
        }
        $this->prepareRows($elements);
        parent::__construct($elements);
    }

    public function filter(\Closure $p){
        $filtered = new Dataset(array_filter($this->toArray(), $p));
        return $filtered->isEmpty() ? false : $filtered;
    }

    /**
     * @param array $condition Array chave valor (Coluna === valor), ou valor pode ser um array (Coluna IN(valor1,valor2,valor3...)). Ex.: (array("CDFILIAL"=>"9999")) , (array("CDFILIAL"=>array("9999","8888",...)))
     * @return bool(false)|\Geral\Util\Dataset
     */
    public function find(array $condition)
    {
        $filterFunc = function($row) use($condition){
            $return = array();
            foreach($condition as $col => $val){
                if(is_array($val)){
                    if(isset($row[$col]) && in_array($row[$col], $val))
                        $return[] = true;
                }else{
                    if(isset($row[$col]) && $row[$col] == $val)
                        $return[] = true;
                }
            }

            return count($return) === count($condition);
        };
        $filtered = new Dataset(array_filter($this->toArray(), $filterFunc));
        return $filtered->isEmpty() ? false : $filtered;
    }

    protected function prepareRows(&$elements)
    {

        $sortFunc = function($a, $b) {
            if ($a["ID"] > $b["ID"])
                return 1;
            if ($a["ID"] < $b["ID"])
                return -1;
            if ($a["ID"] == $b["ID"])
                return 0;
            return false;
        };

        usort($elements, $sortFunc);
    }

    public function delete($row)
    {
        $this->deleteRow($row);
    }

    public function persist($row)
    {
        if (isset($row["_STATE"])) {
            if ($row["_STATE"] === "U")
                $this->updateRow($row);
            else
                $this->insertRow($row);
        }
    }

    public function flush()
    {
        $dataset = &$this;
        $removeState = function($key, $element) use($dataset){
                unset($element["_ID"]);
                unset($element["_STATE"]);
                $dataset->set($key, $element);
                return true;
        };
        $this->forAll($removeState);
        if (!file_put_contents($this->filepath, json_encode($this->toArray()))) {
            throw new \Exception("Falha ao salvar registro.");
        }
    }

    protected function updateRow($row)
    {
        $iterator = $this->getIterator();
        while ($iterator->valid()) {
            $current = $iterator->current();
            if ($current["ID"] === $row["ID"]) {
                $this->set($this->key(), $row);
                break;
            }
            $iterator->next();
        }
    }

    protected function insertRow($row)
    {
        $this->add($row);
    }

    protected function deleteRow($row) {
        $iterator = $this->getIterator();
        while ($iterator->valid()) {
            $current = $iterator->current();
            if ($current["ID"] === $row["ID"]) {
                $this->remove($this->key());
                break;
            }
            $iterator->next();
        }
    }
}